﻿using System;
using System.Collections.Generic;
using System.Linq;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;
using Verse;
using Verse.AI;
using Verse.AI.Group;
using WhatTheHack.Comps;

namespace WhatTheHack.Buildings;

public class Building_RogueAI : Building
{
    public enum Mood : byte
    {
        Happy = 0,
        Annoyed = 1,
        Mad = 2
    }

    private const int MAXCONTROLLABLEMECHS = 6;
    private const int MAXCONTROLLABLETURRETS = 4;
    private const int MAXHACKABLE = 2;
    private const int NUMTEXTSHAPPY = 50;
    private const int NUMTEXTSANNOYED = 50;
    private const int NUMTEXTSMAD = 15;
    private const int NUMTEXTSGOROGUE = 15;
    private const int NUMTEXTSSIGNALFROMEARTH = 5;
    private const int NUMTEXTSSIGNALFROMEARTHRESPONSE = 5;

    private const int MINLEVELMANAGEPOWER = 2;
    private const int MINLEVELCONTROLTURRET = 3;
    private const int MINLEVELCONTROLMECH = 4;
    private const int MINLEVELHACKMECH = 5;
    private const int MINTEXTTIMEOUT = 6;
    private const float TEXTDURATION = 4f;

    private readonly float moodDrainCtrlMech = 0.075f;
    private readonly float moodDrainCtrlTur = 0.075f;
    private readonly float moodDrainDamage = 2.0f;
    private readonly float moodDrainForceTalkGibberish = 5.0f;
    private readonly float moodDrainHack = 0.3f;
    private readonly float moodDrainNoPower = 0.2f;
    private readonly List<ActionItem> queuedActions = new List<ActionItem>();
    private int abilityWarmUpTicks;

    public List<Pawn> controlledMechs = new List<Pawn>();

    public List<Building_TurretGun> controlledTurrets = new List<Building_TurretGun>();
    private int currentAbilityTicksTotal;
    public bool goingRogue;
    public List<Pawn> hackedMechs = new List<Pawn>();
    private bool isConscious;
    public bool managingPowerNetwork;
    public float moodDrainPreventZzztt = 0.5f;
    public List<Pawn> rogueMechs = new List<Pawn>();
    public List<Building_TurretGun> rogueTurrets = new List<Building_TurretGun>();
    private int textTimeout;

    private bool warnedPlayerAboutMood = false;

    public bool IsConscious
    {
        get => isConscious;
        set
        {
            isConscious = value;
            if (!isConscious)
            {
                return;
            }

            OverlayComp.SetLookAround();
            DrainMood(50);
            OutputText("WTH_RogueAI_HelloWorld".Translate());
            LessonAutoActivator.TeachOpportunity(WTH_DefOf.WTH_Concept_RogueAI_LevelUp, OpportunityType.Important);
        }
    }

    public bool WarmingUpAbility => abilityWarmUpTicks > 0;


    public CompRefuelable RefuelableComp => GetComp<CompRefuelable>();

    private CompDataLevel DataLevelComp => GetComp<CompDataLevel>();

    public CompPowerPlant_RogueAI PowerPlantComp => PowerComp as CompPowerPlant_RogueAI;

    private CompOverlay OverlayComp => GetComp<CompOverlay>();

    public Mood CurMoodCategory
    {
        get
        {
            //CompRefuelable compRefuelable = GetComp<CompRefuelable>();
            if (RefuelableComp.FuelPercentOfMax < 0.25f)
            {
                return Mood.Mad;
            }

            if (!(RefuelableComp.FuelPercentOfMax < 0.45f))
            {
                return Mood.Happy;
            }

            LessonAutoActivator.TeachOpportunity(WTH_DefOf.WTH_Concept_RogueAI_Mood, OpportunityType.Important);
            return Mood.Annoyed;
        }
    }

    public bool CurrentlyDrainingMood =>
        hackedMechs.Count > 0 || controlledMechs.Count > 0 || controlledTurrets.Count > 0;

    public override void SpawnSetup(Map map, bool respawningAfterLoad)
    {
        base.SpawnSetup(map, respawningAfterLoad);
        UpdateGlower();
        OutputText("WTH_RogueAI_HelloWorld".Translate());
        if (!goingRogue)
        {
            PowerPlantComp.overcharging = false; //Fix for saves affected by previous issue
        }

        LessonAutoActivator.TeachOpportunity(WTH_DefOf.WTH_Concept_RogueAI, OpportunityType.Important);
    }

    public override void Destroy(DestroyMode mode = DestroyMode.Vanish)
    {
        CancelLinks();
        StopGoingRogue();
        foreach (var beacon in Map.listerBuildings.allBuildingsColonist.Where(b => b is Building_MechanoidBeacon)
                     .Cast<Building_MechanoidBeacon>())
        {
            beacon.CancelStartup();
        }

        base.Destroy(mode);
    }

    public override void Tick()
    {
        base.Tick();
        TickActionQueue();
        if (this.IsHashIntervalTick(250))
        {
            TickRare();
        }

        if (abilityWarmUpTicks <= 0)
        {
            return;
        }

        DrawWarmup();
        abilityWarmUpTicks--;
        /*
        if (this.RefuelableComp.Fuel < Base.moodAutoDeactivate)
        {
            CancelLinks();
            Messages.Message("WTH_Message_MoodAutoDeactivate".Translate(), new RimWorld.Planet.GlobalTargetInfo(this.Position, this.Map), MessageTypeDefOf.PositiveEvent);
        }
        */
    }

    public override void TickRare()
    {
        MaybeTalkGibberish();
        DrainMoodAfterTickRare();
        if (CurMoodCategory == Mood.Mad && !goingRogue)
        {
            MaybeGoRogue();
        }

        if (textTimeout > 0)
        {
            textTimeout--;
        }

        foreach (var turret in controlledTurrets)
        {
            if (turret.DestroyedOrNull())
            {
                CancelControlTurret(turret);
            }
        }

        var store = Base.Instance.GetExtendedDataStorage();
        if (store == null)
        {
            return;
        }

        var mapData = store.GetExtendedDataFor(Map);
        mapData.rogueAI = this;
    }

    private void MaybeTalkGibberish()
    {
        var textChance = goingRogue ? 1.0f : 0.02f;
        if (Rand.Chance(textChance))
        {
            TalkGibberish();
        }
    }

    private void TalkGibberish()
    {
        if (!IsConscious)
        {
            return;
        }

        var randomColonistName = Map.mapPawns.FreeColonists.RandomElement().Name.ToStringShort;
        string signalFromEarth =
            ("WTH_RogueAI_SignalFromEarth_" + Rand.RangeInclusive(0, NUMTEXTSSIGNALFROMEARTH - 1)).Translate();
        string signalFromEarthResponse =
            ("WTH_RogueAI_SignalFromEarth_Response_" + Rand.RangeInclusive(0, NUMTEXTSSIGNALFROMEARTHRESPONSE - 1))
            .Translate();
        string text;

        if (goingRogue)
        {
            text = $"WTH_RogueAI_GoRogue_Remark_{Rand.RangeInclusive(0, NUMTEXTSGOROGUE - 1)}";
        }
        else if (CurMoodCategory == Mood.Happy)
        {
            text = $"WTH_RogueAI_Happy_Remark_{Rand.RangeInclusive(0, NUMTEXTSHAPPY - 1)}";
        }
        else if (CurMoodCategory == Mood.Annoyed)
        {
            text = $"WTH_RogueAI_Annoyed_Remark_{Rand.RangeInclusive(0, NUMTEXTSANNOYED - 1)}";
        }
        else
        {
            text = $"WTH_RogueAI_Mad_Remark_{Rand.RangeInclusive(0, NUMTEXTSMAD - 1)}";
        }

        OutputText(text.Translate(randomColonistName, signalFromEarth, signalFromEarthResponse));
    }

    private void OutputText(string text, bool force = false)
    {
        if (!force && textTimeout > 0 || !IsConscious)
        {
            return;
        }

        var color = Color.white;
        if (goingRogue)
        {
            color = Color.red;
        }

        var z = 1.75f;
        if (force)
        {
            z -= 0.5f; //Make sure that when text output is forced, the texts won't overlap
        }

        Utilities.ThrowStaticText(DrawPos + new Vector3(0, 0, z), Map, text, color, TEXTDURATION);
        textTimeout += MINTEXTTIMEOUT;
    }

    private void MaybeGoRogue()
    {
        //change to go rogue at tick rare gets increasingly large, starting at 1/50; 
        var goRogueChance = 1 / (RefuelableComp.Fuel * 2.5f);
        if (!Rand.Chance(goRogueChance))
        {
            return;
        }

        Find.LetterStack.ReceiveLetter("WTH_Message_GoRogue_Label".Translate(),
            "WTH_Message_GoRogue_Description".Translate(), LetterDefOf.ThreatBig, new GlobalTargetInfo(this));
        GoRogue();
    }

    public override void PostApplyDamage(DamageInfo dinfo, float totalDamageDealt)
    {
        base.PostApplyDamage(dinfo, totalDamageDealt);
        DrainMood(moodDrainDamage);
        OutputText("WTH_RogueAI_Hurt_Remark".Translate());
    }

    private void DrainMoodAfterTickRare()
    {
        foreach (var unused in controlledMechs)
        {
            DrainMood(moodDrainCtrlMech);
        }

        foreach (var unused in controlledTurrets)
        {
            DrainMood(moodDrainCtrlTur);
        }

        foreach (var unused in hackedMechs)
        {
            DrainMood(moodDrainHack);
        }

        if (PowerPlantComp.PowerOn)
        {
            return;
        }

        DrainMood(moodDrainNoPower);
        OutputText("WTH_RogueAI_NoPower_Remark".Translate());
    }

    public void DrainMood(float amount)
    {
        //Mood oldMood = CurMoodCategory;
        if (!isConscious || goingRogue)
        {
            return;
        }

        var minFuel = 1f;
        if (RefuelableComp.Fuel - amount < minFuel)
        {
            amount = RefuelableComp.Fuel - minFuel;
        }

        RefuelableComp.ConsumeFuel(amount);
        UpdateGlower();
    }

    public override IEnumerable<Gizmo> GetGizmos()
    {
        foreach (var gizmo in base.GetGizmos())
        {
            yield return gizmo;
        }

        yield return GetTalkGibberishGizmo();
        yield return GetManagePowerNetworkGizmo();
        yield return GetControlTurretAvtivateGizmo();
        yield return GetControlMechanoidActivateGizmo();
        yield return GetHackingAvtivateGizmo();

        foreach (var mech in controlledMechs)
        {
            yield return GetControlMechanoidCancelGizmo(mech);
        }

        var index = 1;

        foreach (var turret in controlledTurrets)
        {
            yield return GetControlTurretCancelGizmo(turret, index);
            index++;
        }

        index = 1;

        foreach (var mech in hackedMechs)
        {
            yield return GetHackingCancelGizmo(mech, index);
            index++;
        }
    }

    public bool GlobalShouldDisable(out string reason)
    {
        reason = "";
        var result = false;
        if (!isConscious)
        {
            reason += "\n- " + "WTH_Reason_NotConscious".Translate();
            result = true;
        }

        if (abilityWarmUpTicks > 0)
        {
            reason += "\n- " + "WTH_Reason_WarmingUp".Translate();
            result = true;
        }

        if (!PowerPlantComp.PowerOn)
        {
            reason += "\n- " + "WTH_Reason_NoPower".Translate();
            result = true;
        }

        if (!goingRogue)
        {
            return result;
        }

        reason = "\n- " + "WTH_Reason_GoingRogue".Translate();

        return true;
    }

    private void TickActionQueue()
    {
        foreach (var actionItem in queuedActions)
        {
            actionItem.Tick();
        }

        queuedActions.RemoveAll(i => i.shouldClean);
    }

    private void DrawWarmup()
    {
        var height = 0.5f;
        var width = 0.25f * (1 - (abilityWarmUpTicks / (float)currentAbilityTicksTotal));
        //GenDraw.DrawCooldownCircle(this.Position.ToVector3Shifted() + new Vector3(0, 3f, 0), width);

        var s = new Vector3(width, 10f, height);
        var matrix = default(Matrix4x4);
        matrix.SetTRS(Position.ToVector3Shifted() + new Vector3(0, 3f, 0), Quaternion.identity, s);
        Graphics.DrawMesh(MeshPool.plane20, matrix,
            SolidColorMaterials.SimpleSolidColorMaterial(new Color(1f, 1f, 1f, 0.5f)), 0);
    }

    private void DoAbility(Action action, int warmupTime)
    {
        abilityWarmUpTicks = warmupTime;
        currentAbilityTicksTotal = warmupTime;
        queuedActions.Add(new ActionItem(action, warmupTime));
    }

    private Gizmo GetTalkGibberishGizmo()
    {
        var command = new Command_Action
        {
            defaultLabel = "WTH_Gizmo_TalkGibberish_Label".Translate(),
            defaultDesc = "WTH_Gizmo_TalkGibberish_Description".Translate(moodDrainForceTalkGibberish),
            icon = ContentFinder<Texture2D>.Get("UI/RogueAI_TalkGibberish"),
            disabled = GlobalShouldDisable(out var reason),
            disabledReason = reason
        };
        if (textTimeout > 0)
        {
            command.disabled = true;
            command.disabledReason = "\n- " + "WTH_Reason_TalkedTooRecently".Translate();
        }

        command.action = delegate
        {
            DoAbility(TalkGibberish, 250);
            DrainMood(moodDrainForceTalkGibberish);
        };
        return command;
    }

    private Gizmo GetManagePowerNetworkGizmo()
    {
        var command = new Command_Toggle
        {
            defaultLabel = "WTH_Gizmo_ManagePowerNetwork_Label".Translate(),
            defaultDesc = "WTH_Gizmo_ManagePowerNetwork_Description".Translate(),
            icon = ContentFinder<Texture2D>.Get("UI/RogueAI_Manage_Network"),
            disabled = GlobalShouldDisable(out var reason),
            disabledReason = reason
        };
        if (DataLevelComp.curLevel < MINLEVELMANAGEPOWER)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_LevelInsufficient".Translate(MINLEVELMANAGEPOWER);
        }

        command.isActive = () => managingPowerNetwork;
        command.toggleAction = delegate
        {
            managingPowerNetwork = !managingPowerNetwork;
            //GoRogue();
        };
        return command;
    }

    private Gizmo GetControlMechanoidCancelGizmo(Pawn mech)
    {
        var command = new Command_Action_Highlight
        {
            defaultLabel = mech.Name.ToStringShort,
            defaultDesc = "WTH_Gizmo_ControlMechanoidCancel_Description".Translate(),
            parent = this,
            thing = mech
        };

        var iconFound = Base.Instance.cancelControlMechTextures.TryGetValue(mech.def.defName, out var icon);
        if (iconFound)
        {
            command.icon = icon;
        }

        command.action = delegate { CancelControlMechanoid(mech); };
        return command;
    }

    private void CancelControlMechanoid(Pawn mech)
    {
        var mechData = Base.Instance.GetExtendedDataStorage().GetExtendedDataFor(mech);
        mechData.controllingAI = null;
        controlledMechs.Remove(mech);
        mech.drafter.Drafted = false;
    }

    private Gizmo GetControlMechanoidActivateGizmo()
    {
        var command = new Command_Target
        {
            defaultLabel = "WTH_Gizmo_ControlMechanoidActivate_Label".Translate(),
            defaultDesc = "WTH_Gizmo_ControlMechanoidActivate_Description".Translate(MAXCONTROLLABLEMECHS),
            targetingParams = GetTargetingParametersForControlling(),
            hotKey = KeyBindingDefOf.Misc5,
            icon = ContentFinder<Texture2D>.Get("UI/RogueAI_Control")
        };

        var shouldDisable = GlobalShouldDisable(out var reason);
        command.disabled = shouldDisable;
        command.disabledReason = reason;
        if (controlledMechs.Count >= MAXCONTROLLABLEMECHS)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_AtMaximum".Translate(MAXCONTROLLABLEMECHS);
        }

        if (DataLevelComp.curLevel < MINLEVELCONTROLMECH)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_LevelInsufficient".Translate(MINLEVELCONTROLMECH);
        }

        command.action = delegate(LocalTargetInfo target)
        {
            if (target is { HasThing: true, Thing: Pawn mech })
            {
                DoAbility(delegate { ControlMechanoid(mech); }, 250);
            }
        };
        return command;
    }

    private void ControlMechanoid(Pawn mech)
    {
        var mechData = Base.Instance.GetExtendedDataStorage().GetExtendedDataFor(mech);
        mechData.controllingAI = this;
        controlledMechs.Add(mech);
        mechData.isActive = true;
        mech.drafter.Drafted = true;
    }

    private static TargetingParameters GetTargetingParametersForControlling()
    {
        return new TargetingParameters
        {
            canTargetPawns = true,
            canTargetBuildings = false,
            mapObjectTargetsMustBeAutoAttackable = false,
            validator = delegate(TargetInfo targ)
            {
                if (!targ.HasThing)
                {
                    return false;
                }

                var pawn = targ.Thing as Pawn;
                return pawn is { Downed: false } && pawn.IsHacked();
            }
        };
    }

    private Gizmo GetHackingCancelGizmo(Pawn mech, int index)
    {
        var command = new Command_Action_Highlight
        {
            defaultLabel = "WTH_Gizmo_HackingCancel_Label".Translate() + " " + index,
            defaultDesc = "WTH_Gizmo_HackingCancel_Description".Translate(),
            parent = this,
            thing = mech
        };

        var iconFound = Base.Instance.cancelControlMechTextures.TryGetValue(mech.def.defName, out var icon);
        if (iconFound)
        {
            command.icon = icon;
        }

        command.action = delegate { CancelHacking(mech); };
        return command;
    }

    private void CancelHacking(Pawn mech)
    {
        var mechData = Base.Instance.GetExtendedDataStorage().GetExtendedDataFor(mech);
        mechData.controllingAI = null;
        mech.RevertToFaction(mechData.originalFaction);
        mech.drafter.Drafted = false;
        hackedMechs.Remove(mech);
    }

    private Gizmo GetHackingAvtivateGizmo()
    {
        var command = new Command_Target
        {
            defaultLabel = "WTH_Gizmo_HackingAvtivate_Label".Translate(),
            defaultDesc = "WTH_Gizmo_HackingAvtivate_Description".Translate(MAXHACKABLE),
            targetingParams = GetTargetingParametersForHacking(),
            hotKey = KeyBindingDefOf.Misc5,
            icon = ContentFinder<Texture2D>.Get("UI/RogueAI_Hack")
        };
        var shouldDisable = GlobalShouldDisable(out var reason);
        command.disabled = shouldDisable;
        command.disabledReason = reason;
        if (hackedMechs.Count >= MAXHACKABLE)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_AtMaximum".Translate(MAXHACKABLE);
        }

        if (DataLevelComp.curLevel < MINLEVELHACKMECH)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_LevelInsufficient".Translate(MINLEVELHACKMECH);
        }

        command.action = delegate(LocalTargetInfo target)
        {
            if (target is { HasThing: true, Thing: Pawn mech })
            {
                DoAbility(delegate { HackMech(mech); }, 600);
            }
        };
        return command;
    }

    private void HackMech(Pawn mech)
    {
        var mechData = Base.Instance.GetExtendedDataStorage().GetExtendedDataFor(mech);
        mechData.originalFaction = mech.Faction;
        mechData.controllingAI = this;
        hackedMechs.Add(mech);
        mech.SetFaction(Faction.OfPlayer);
        mech.story = new Pawn_StoryTracker(mech);
        mech.jobs.EndCurrentJob(JobCondition.InterruptForced);
        mech.drafter = new Pawn_DraftController(mech)
        {
            Drafted = true
        };
    }

    private static TargetingParameters GetTargetingParametersForHacking()
    {
        return new TargetingParameters
        {
            canTargetPawns = true,
            canTargetBuildings = false,
            mapObjectTargetsMustBeAutoAttackable = false,
            validator = delegate(TargetInfo targ)
            {
                if (!targ.HasThing)
                {
                    return false;
                }

                var pawn = targ.Thing as Pawn;
                return pawn is { Downed: false } && pawn.IsMechanoid() && pawn.Faction != Faction.OfPlayer;
            }
        };
    }

    private Gizmo GetControlTurretCancelGizmo(Building_TurretGun turret, int index)
    {
        var command = new Command_Action_Highlight
        {
            defaultLabel = "WTH_Gizmo_ControlTurretCancel_Label".Translate() + index,
            defaultDesc = "WTH_Gizmo_ControlTurretCancel_Description".Translate(),
            parent = this,
            thing = turret
        };

        var iconFound = Base.Instance.cancelControlTurretTextures.TryGetValue(turret.def.defName, out var icon);
        if (iconFound)
        {
            command.icon = icon;
        }

        command.action = delegate { CancelControlTurret(turret); };
        return command;
    }

    private void CancelControlTurret(Building_TurretGun turret)
    {
        controlledTurrets.Remove(turret);
    }

    private Gizmo GetControlTurretAvtivateGizmo()
    {
        var command = new Command_Target
        {
            defaultLabel = "WTH_Gizmo_ControlTurretActivate_Label".Translate(),
            defaultDesc = "WTH_Gizmo_ControlTurretActivate_Description".Translate(MAXCONTROLLABLETURRETS),
            targetingParams = GetTargetingParametersForControlTurret(),
            icon = ContentFinder<Texture2D>.Get("UI/RogueAI_Control_Turret")
        };
        var shouldDisable = GlobalShouldDisable(out var reason);
        command.disabled = shouldDisable;
        command.disabledReason = reason;
        if (controlledTurrets.Count >= MAXCONTROLLABLETURRETS)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_AtMaximum".Translate(MAXCONTROLLABLETURRETS);
        }

        if (DataLevelComp.curLevel < MINLEVELCONTROLTURRET)
        {
            command.disabled = true;
            command.disabledReason += "\n- " + "WTH_Reason_LevelInsufficient".Translate(MINLEVELCONTROLTURRET);
        }

        command.action = delegate(LocalTargetInfo target)
        {
            if (target is { HasThing: true, Thing: Building_TurretGun turret })
            {
                DoAbility(delegate { controlledTurrets.Add(turret); }, 250);
            }
        };
        return command;
    }

    private static TargetingParameters GetTargetingParametersForControlTurret()
    {
        return new TargetingParameters
        {
            canTargetPawns = false,
            canTargetBuildings = true,
            mapObjectTargetsMustBeAutoAttackable = false,
            validator = delegate(TargetInfo targ)
            {
                if (!targ.HasThing)
                {
                    return false;
                }

                var building = targ.Thing as Building;
                return building is Building_TurretGun turret
                       && turret.Faction == Faction.OfPlayer
                       && turret.mannableComp == null;
            }
        };
    }

    public void GoRogue()
    {
        OverlayComp.UnsetLookAround();
        goingRogue = true;
        SetFaction(Faction.OfMechanoids);
        UpdateGlower();
        CancelLinks();
        QueueGoRogueActions();
    }

    private void QueueGoRogueActions()
    {
        var possibleActions = new List<Action>();

        void CauseZzzttAction()
        {
            GoRogue_CauseZzztts();
        }

        void HackTurretsAction()
        {
            GoRogue_HackTurrets();
        }

        void HackMechsAction()
        {
            GoRogue_HackMechs();
        }

        if (DataLevelComp.curLevel >= MINLEVELMANAGEPOWER)
        {
            possibleActions.Add(CauseZzzttAction);
            possibleActions.Add(CauseZzzttAction);
            possibleActions.Add(CauseZzzttAction);
        }

        if (DataLevelComp.curLevel >= MINLEVELCONTROLTURRET)
        {
            possibleActions.Add(HackTurretsAction);
            possibleActions.Add(HackTurretsAction);
        }

        if (DataLevelComp.curLevel >= MINLEVELHACKMECH)
        {
            possibleActions.Add(HackMechsAction);
            possibleActions.Add(HackMechsAction);
        }

        var duration = Rand.Range(10000, 15000);
        var actionInterval =
            (duration + 4000) /
            (possibleActions.Count + 1); //add 4000 to duration so not every action can happen during the duration. 
        var actionDelay = 10;
        while (!possibleActions.NullOrEmpty())
        {
            var action = possibleActions.RandomElement();
            queuedActions.Add(new ActionItem(action, actionDelay));
            possibleActions.Remove(action);
            actionDelay += actionInterval;
        }

        queuedActions.Add(new ActionItem(
            StopGoingRogue,
            duration
        ));
    }

    private void CancelLinks()
    {
        while (controlledMechs.Count > 0)
        {
            CancelControlMechanoid(controlledMechs.Last());
        }

        while (controlledTurrets.Count > 0)
        {
            CancelControlTurret(controlledTurrets.Last());
        }

        while (hackedMechs.Count > 0)
        {
            CancelHacking(hackedMechs.Last());
        }
    }

    public void StopGoingRogue()
    {
        if (!goingRogue)
        {
            return;
        }

        goingRogue = false;
        UpdateGlower();
        OverlayComp.SetLookAround();
        RefuelableComp.fuel = 30f;
        //Traverse.Create(RefuelableComp).Field("fuel").SetValue(30f);
        SetFaction(Faction.OfPlayer);
        while (rogueMechs.Count > 0)
        {
            var rogueMech = rogueMechs.Last();
            rogueMech.SetFaction(Faction.OfPlayer);
            rogueMechs.Remove(rogueMech);
            if (rogueMech.relations == null)
            {
                rogueMech.relations = new Pawn_RelationsTracker(rogueMech);
            }

            if (rogueMech.story == null)
            {
                rogueMech.story = new Pawn_StoryTracker(rogueMech);
            }
        }

        while (rogueTurrets.Count > 0)
        {
            var turret = rogueTurrets.Last();
            turret.SetFaction(Faction.OfPlayer);
            rogueTurrets.Remove(turret);
        }

        queuedActions.Clear();
        PowerPlantComp.overcharging = false;
        OutputText("WTH_RogueAI_StopGoingRogue".Translate(), true);
        Messages.Message("WTH_Message_RogueAI_StopGoingRogue".Translate(), new GlobalTargetInfo(Position, Map),
            MessageTypeDefOf.PositiveEvent);
    }

    public void UpdateGlower()
    {
        var glowerComp = GetComp<CompGlower>();
        var mood = CurMoodCategory;
        glowerComp.Props.glowRadius = 4f;
        if (!IsConscious)
        {
            glowerComp.Props.glowRadius = 0f;
            //Don't set any color;
        }
        else if (goingRogue)
        {
            glowerComp.Props.glowColor = new ColorInt(255, 0, 0, 0);
            glowerComp.Props.glowRadius = 12f;
        }
        else if (mood == Mood.Happy)
        {
            glowerComp.Props.glowColor = new ColorInt(35, 152, 255, 0);
        }
        else if (mood == Mood.Annoyed)
        {
            glowerComp.Props.glowColor = new ColorInt(255, 119, 35, 0);
        }
        else
        {
            glowerComp.Props.glowColor = new ColorInt(255, 0, 0, 0);
        }

        glowerComp.UpdateLit(Map);
        Map.glowGrid.MarkGlowGridDirty(Position);
    }

    private void GoRogue_HackMechs()
    {
        var shouldHack = Map.mapPawns.SpawnedPawnsInFaction(Faction.OfPlayer).Where(p => p.IsHacked() && !p.Downed)
            .ToList();
        var nShouldHack = Rand.Range(1, 4);
        var nHacked = 0;
        while (nShouldHack > 0 && shouldHack.Count > 0)
        {
            var mech = shouldHack.RandomElement();
            mech.RevertToFaction(Faction.OfMechanoids);
            mech.jobs.EndCurrentJob(JobCondition.InterruptForced);
            if (mech.GetLord() == null || mech.GetLord().LordJob == null)
            {
                LordMaker.MakeNewLord(Faction.OfMechanoids, new LordJob_AssaultColony(Faction.OfMechanoids), mech.Map,
                    new List<Pawn> { mech });
            }

            nShouldHack--;
            shouldHack.Remove(mech);
            rogueMechs.Add(mech);
            nHacked++;
        }

        if (nHacked > 0)
        {
            Find.LetterStack.ReceiveLetter("WTH_Message_GoRogue_HackMechs_Label".Translate(),
                "WTH_Message_GoRogue_HackMechs_Description".Translate(), LetterDefOf.ThreatBig,
                new LookTargets(rogueMechs));
        }
    }

    private void GoRogue_HackTurrets()
    {
        bool IsSuitableTurret(Thing t)
        {
            return t is Building_TurretGun turret && turret.Faction == Faction.OfPlayer && turret.mannableComp == null;
        }

        //Traverse.Create(turret).Field("mannableComp").GetValue<CompMannable>() == null;
        var turrets = Map.spawnedThings.Where(IsSuitableTurret).Cast<Building_TurretGun>().ToList();
        var nShouldHack = Rand.Range(1, 3);
        var nHacked = 0;
        while (nShouldHack > 0 && turrets.Count > 0)
        {
            var turret = turrets.RandomElement();
            turret.SetFaction(Faction.OfMechanoids);
            nShouldHack--;
            turrets.Remove(turret);
            rogueTurrets.Add(turret);
            nHacked++;
        }

        if (nHacked > 0)
        {
            Find.LetterStack.ReceiveLetter("WTH_Message_GoRogue_HackTurrets_Label".Translate(),
                "WTH_Message_GoRogue_HackTurrets_Description".Translate(), LetterDefOf.ThreatBig,
                new LookTargets(rogueTurrets.Cast<Thing>()));
        }
    }

    private void GoRogue_CauseZzztts()
    {
        PowerPlantComp.overcharging = true;
        var potentialTargets = ShortCircuitUtility.GetShortCircuitablePowerConduits(Map);
        var numZzztts = Rand.Range(3, 5);
        var rareTicksUntilAction = 0;
        for (var i = 0; i < numZzztts; i++)
        {
            if (!potentialTargets.TryRandomElement(out var potentialTarget))
            {
                continue;
            }

            queuedActions.Add(new ActionItem(
                delegate { ShortCircuitUtility.DoShortCircuit(potentialTarget); },
                rareTicksUntilAction)
            );
            rareTicksUntilAction += Rand.Range(500, 1000);
        }

        queuedActions.Add(new ActionItem(
            delegate { PowerPlantComp.overcharging = false; },
            rareTicksUntilAction)
        );

        Find.LetterStack.ReceiveLetter("WTH_Message_GoRogue_CauseZzztts_Label".Translate(),
            "WTH_Message_GoRogue_CauseZzztts_Description".Translate(), LetterDefOf.ThreatBig,
            new GlobalTargetInfo(this));
    }

    public override void ExposeData()
    {
        base.ExposeData();
        Scribe_Values.Look(ref textTimeout, "textTimeout");
        Scribe_Values.Look(ref isConscious, "isConscious");
        Scribe_Values.Look(ref goingRogue, "goingRogue");
        Scribe_Values.Look(ref managingPowerNetwork, "managingPowerNetwork");
        Scribe_Values.Look(ref abilityWarmUpTicks, "abilityCooldownTicks");
        Scribe_Values.Look(ref currentAbilityTicksTotal, "currentAbilityTicksTotal");
        Scribe_Collections.Look(ref controlledMechs, "controlledMechs", LookMode.Reference);
        Scribe_Collections.Look(ref hackedMechs, "hackedMechs", LookMode.Reference);
        Scribe_Collections.Look(ref controlledTurrets, "controlledTurrets", LookMode.Reference);
        Scribe_Collections.Look(ref rogueMechs, "rogueMechs", LookMode.Reference);
        Scribe_Collections.Look(ref rogueTurrets, "rogueTurrets", LookMode.Reference);
    }

    private class Command_Action_Highlight : Command_Action
    {
        public Building_RogueAI parent;
        public Thing thing;

        public override void GizmoUpdateOnMouseover()
        {
            base.GizmoUpdateOnMouseover();
            GenDraw.DrawLineBetween(parent.Position.ToVector3Shifted(), thing.Position.ToVector3Shifted(),
                SimpleColor.White);
        }
    }

    private class ActionItem
    {
        public readonly Action action;
        public bool shouldClean;
        public int ticksUntilAction;

        public ActionItem(Action action, int tickUntilAction)
        {
            this.action = action;
            ticksUntilAction = tickUntilAction;
        }

        public void Tick()
        {
            ticksUntilAction--;
            if (ticksUntilAction != 0)
            {
                return;
            }

            action.Invoke();
            shouldClean = true;
        }
    }
}